/*******************************************************************
 *
 *	File:		Dialogs.h
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *      Modified for Series 60 by Staffan Ulfberg (staffan@ulfberg.se)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __DIALOGS_H
#define __DIALOGS_H

#include "target.h"

#include <eikenv.h>
#include <eikdialg.h>
#include <eiktxlbx.h>
#include <eiktxlbm.h>
#include <eiklabel.h>
#include <eikedwob.h>
#ifdef __SERIES60__
#include <akndialog.h>
#include <aknselectionlist.h>
#include <aknsettingitemlist.h>
#include <aknprogressdialog.h>
#else
#include <eikchlst.h>
#include <eikchkbx.h>
#include <eikprogi.h>
#endif
#ifdef __UIQ__
#include <qiknumbereditor.h>
#include <eikdlgtb.h>
#endif

#include <eemame.rsg>
#include "EmameGlobal.h"

class CMameEngine;
class CInstalledGameList;
class CRichText;
class CEikRichTextEditor;



/************************************************
 *
 * GameRunning Dialog
 *
 ************************************************/


#ifdef __SERIES60__
class CGameRunningDialog : public CAknDialog
#else
class CGameRunningDialog : public CEikDialog
#endif
{
public:
    CGameRunningDialog(const TDesC8& aGameName, CMameEngine& aMameEngine);
    
  protected:
    void PreLayoutDynInitL();
    
#if defined __SERIES60__ || defined __SERIES80__ || defined __NOKIA7710__
    TBool OkToExitL(TInt aCommandId);
#elif defined __UIQ__
    void HandleControlStateChangeL(TInt aControlId);
#endif
    
    TBuf<64> iGameName;
    CMameEngine &iMameEngine;
};


/************************************************
 *
 * Scanning ROMs Dialog
 *
 ************************************************/

class CScanningRomDialog : 
#ifdef __SERIES60__
                           public CAknProgressDialog
#else
                           public CEikDialog
#endif
{
  public:
    CScanningRomDialog(CMameEngine& aMameEngine, 
                       CInstalledGameList& aGameList);
    ~CScanningRomDialog();

  protected: 
    void PreLayoutDynInitL();

    static TInt IdleCallBack(TAny*);
    TInt DoUpdate();

    CEikDialog *iThis;
    CEikProgressInfo *iProgressInfo;
    TInt iGameNo;
    TInt iNoOfGames;
    CMameEngine &iMameEngine;
    CIdle *iIdle;
    CInstalledGameList &iInstalledGameList;
};


/************************************************
 *
 * Game Options "Dialog"
 *
 ************************************************/

#ifdef __SERIES60__
class COptionsListView : public CAknSettingItemList
{
  public:
    COptionsListView(TGameOptions gameOptions);
    ~COptionsListView();
    void CreateL();
    CAknSettingItem *CreateSettingItemL(TInt id);
    TGameOptions GameOptions();
    
  private:
    TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
    void SizeChanged();
    
  private:
    TGameOptions iGameOptions;
};
#else
class CGameOptionsDialog : public CEikDialog
{
 public:
  CGameOptionsDialog(TGameOptions& aGameOptions) 
    : iGameOptions(aGameOptions)
    {
    }
  
 protected:
  void PreLayoutDynInitL();
  void SetSizeAndPositionL(const TSize& aSize);
  TBool OkToExitL(TInt aKeycode);
  TGameOptions &iGameOptions;
};
#endif


/************************************************
 *
 * CSimpleTextInfo Dialog
 *
 ************************************************/

class CSimpleTextInfoDialog : 
#ifdef __SERIES60__ 
                              public CAknDialog,
#else
                              public CEikDialog,
#endif
                              public MEikEdwinSizeObserver
{
  public:
    CSimpleTextInfoDialog(TInt aTextIdOne = -1, 
                          TBool aSimpleTextResIdOneIsArray = EFalse);

    virtual TBool HandleEdwinSizeEventL(CEikEdwin* aEdwin,
                                        TEdwinSizeEvent aEventType,
                                        TSize aDesirableEdwinSize);
    
  protected:
    void PreLayoutDynInitL();
    void PostLayoutDynInitL();
    
    virtual void ShowTextL(CRichText& aRichText);
    void ShowSimpleTextL(const TDesC& aSimpleText, CRichText& aRichText);
    
    TInt	iSimpleTextResId;
    TInt	iRichTextCtlId;
    TBool	iSimpleTextResIdIsArray;
};

/************************************************
 *
 * Game Info Dialog
 *
 ************************************************/

class CGameInfoDialog : public CSimpleTextInfoDialog
{
  public:
    CGameInfoDialog(CMameEngine& aMameEngine, TInt aGame);
    
  protected:
    void ShowTextL(CRichText& aRichText);

  private:
    void AppendTextL(TDes& aOutput, TInt aResId, const TDesC& aGameText);

    CMameEngine &iMameEngine;
    TInt iGame;
};

#endif			/* __DIALOGS_H */
