/*******************************************************************
 *
 *	File:		EMameApp.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *      Modified for Series 60,80,90 by Staffan Ulfberg (staffan@ulfberg.se)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

// #define WRITE_LISTINFO 1
// #define DUMP_GAMES_LIST_TO_FILE 1

#include "dialogs.h"
#include "emameapp.h"

#include <eemame.rsg>

#ifdef __SERIES60__
#include <aknlists.h>
#include <aknsfld.h>
#endif
#ifdef __NOKIA7710__
#include <cknborder.h>
#endif
#ifdef __SERIES60_3RD_EDITION__
#include <eikstart.h>
#endif
#ifdef WRITE_LISTINFO
extern "C" {
#include "info.h"
}
#endif

extern "C" {
#include "audit.h"
}

#ifdef __SERIES60_3RD_EDITION__
_LIT(KLastGameFile, "c:\\Mame\\EEMAME\\lastgame.ini");
#else
_LIT(KLastGameFile, "c:\\system\\apps\\eemame\\lastgame.ini");
#endif

CApaDocument* CMameApplication::CreateDocumentL()
{
	CMameDocument*	doc = new (ELeave) CMameDocument(*this);

	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();

	return doc;
}



CMameDocument::CMameDocument(CEikApplication& aApp)
		: CEikDocument(aApp)
{
}

CMameDocument::~CMameDocument()
{
	delete iMameEngine;
}

void CMameDocument::ConstructL()
{
	iMameEngine = CMameEngine::NewL(CEikonEnv::Static()->FsSession());
}


CEikAppUi* CMameDocument::CreateAppUiL()
{
	return new(ELeave) CMameAppUi;
}


CMameAppUi::~CMameAppUi()
{
  if (iAppView != NULL)
  {
    RemoveFromStack(iAppView);
    delete iAppView;
  }
  delete iGameRunner;
}

void CMameAppUi::ConstructL()
{
  BaseConstructL();
  LoadLastRunningGame();
 
  MameEngine().SetWindowGroupId(CEikonEnv::Static()->RootWin().Identifier());
  iGameRunner = CGameRunner::NewL(*this, MameEngine());

  UnblockKeys();

  iAppView=new (ELeave)CMameAppView(&MameEngine());
  iAppView->ConstructL(ClientRect(), iGameToRun);
#if defined __SERIES60__
  iAppView->SetMopParent(this);
#endif
  AddToStackL(iAppView);

  if (MameEngine().InstalledGameList().NoOfGames() == 0)
  {
    CEikDialog *dialog = new (ELeave)
      CSimpleTextInfoDialog(R_SIMPLE_TEXT_NO_INSTALLED_GAMES);
    dialog->ExecuteLD(R_RICH_TEXT_DISPLAY_DIALOG);
  }
  
#ifdef DUMP_GAMES_LIST_TO_FILE
  DumpGamesListToFile();
#endif
#ifdef WRITE_LISTINFO
  FILE *f = fopen("c:\\mameinfo.dat", "w");
  print_mame_info(f, MameEngine().GameDrivers()); 
  fclose(f);
#endif
}

void
CMameAppUi::UnblockKeys()
{
#ifdef __SERIES60__
  SetKeyBlockMode(ENoKeyBlock);
#endif
}

void CMameAppUi::DisplayHelpDialogL()
{
  CSimpleTextInfoDialog *dialog = 
    new (ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_EMAME_HELP, ETrue);
#if defined __SERIES60__
  dialog->SetMopParent(this);
#endif
  dialog->ExecuteLD(R_RICH_TEXT_DISPLAY_DIALOG);
}


void
CMameAppUi::HandleCommandL(TInt aCommand)
{
  switch (aCommand)
  {
    case ECmdRun:
      if (iAppView->CurrentGameIndex(iGameToRun))
      {
        RunGameL();
      }
      break;

    case ECmdInfo:
    {
      TInt index;
      if (iAppView->CurrentGameIndex(index))
      {
        ShowGameDetailsL(index);
      }
      break;
    }

    case ECmdOptions:
    {
      TInt index;
      if (iAppView->CurrentGameIndex(index))
      {
        const struct GameDriver *game = *(MameEngine().GameDrivers() + index);
        TBuf8<8> text8;
        text8.Copy((const TUint8*)game->name);
        if (MameEngine().GameHasSpecificOptions(text8))
        {
          CEikDialog *dialog = new (ELeave)CEikDialog();
          if (dialog->ExecuteLD(R_DIALOG_OK_TO_DELETE_OPTIONS_QUERY))
          {
            MameEngine().ClearGameOptions(text8);
          }
          else
          {
            break;
          }
        }
      }
      DisplayGameOptionsDialogL(KDefaultOptionsGameName);
      break;
    }

    case ECmdOptionsGameSpecific:
    {
      TInt index;
      if (iAppView->CurrentGameIndex(index))
      {
        const struct GameDriver *game = *(MameEngine().GameDrivers() + index);
        TBuf8<8> text8;
        text8.Copy((const TUint8*)game->name);
        DisplayGameOptionsDialogL(text8);
      }
      break;
    }

#ifdef __SERIES60__
    case ECmdSettingsMenuOk :	
    case ECmdSettingsMenuCancel :	
      GameOptionsDialogFinishedL(aCommand);
      break;
#endif

    case ECmdScanGames:
      ScanInstalledGamesL();
      break;
      
    case ECmdHelp:
      DisplayHelpDialogL();
      break;
      
    case ECmdAbout:
      DisplayAboutDialogL();
      break;
      
    case EEikCmdExit: 
#ifdef __SERIES60__
    case EAknSoftkeyExit:
      SetKeyBlockMode(EDefaultBlockMode);
#endif
      Exit();
      break;
  }
}

void
CMameAppUi::LoadLastRunningGame()
{
  TPckg<TInt> pkg(iGameToRun);
  RFile	file;
  TInt err = file.Open(iCoeEnv->FsSession(), KLastGameFile, 0);
  if (!err)
  {
    file.Read(pkg);
    file.Close();
  }
  else
  {
    iGameToRun = -1;
  }
}

void
CMameAppUi::SaveLastRunningGame()
{
  TPckgC<TInt> pkg(iGameToRun);
  RFile	file;
  TInt err = file.Open(iCoeEnv->FsSession(), KLastGameFile, EFileWrite);
  if (err)
  {
    err = file.Create(iCoeEnv->FsSession(), KLastGameFile, EFileWrite);
  }
  if (!err)
  {
    file.Write(pkg);
    file.Close();
  }
}

void
CMameAppUi::RunGameL()
{
  if (iGameToRun == -1)
  {
    return;
  }

  TBuf8<64> gameName;
  iGameRunner->RunGameL(iGameToRun);
  SaveLastRunningGame();

  gameName.Format(_L8("%s"), MameEngine().GameName(iGameToRun));
  iGameRunningDialog = new(ELeave) CGameRunningDialog(gameName, MameEngine());
  iGameRunningDialog->ExecuteLD(R_DIALOG_RUNNING_GAME);
}

void
CMameAppUi::NotifyGameFinished()
{
  if (iGameRunningDialog)
  {
    delete iGameRunningDialog;
    iGameRunningDialog = NULL;
  }
}

void
CMameAppUi::ShowGameDetailsL(TInt aGame)
{
  CGameInfoDialog *dialog = new CGameInfoDialog(MameEngine(), aGame);
#if defined __SERIES60__
  dialog->SetMopParent(this);
#endif
  dialog->ExecuteLD(R_RICH_TEXT_DISPLAY_DIALOG);
}

void
CMameAppUi::DisplayGameOptionsDialogL(const TDesC8 &aGameName)
{
  TGameOptions gameOptions = MameEngine().GameOptions(aGameName);
  gameOptions.iGameName = aGameName;
  
#ifdef __SERIES60__
  iAppView->MakeVisible(EFalse);
  iAvkonAppUi->Cba()->SetCommandSetL(R_SETTINGS_PAGE_CBA);
  iOptionsListbox = new (ELeave)COptionsListView(gameOptions);
  iOptionsListbox->SetMopParent(this);
  iOptionsListbox->CreateL();
#else
  CEikDialog* dialog = new(ELeave) CGameOptionsDialog(gameOptions);
  if (dialog->ExecuteLD(R_DIALOG_GAME_OPTIONS))
  {
    MameEngine().SetGameOptions(gameOptions);
  }
#endif
}

#ifdef __SERIES60__
void CMameAppUi::GameOptionsDialogFinishedL(TInt aCommand)
{
  if(aCommand == ECmdSettingsMenuOk)
  {
    MameEngine().SetGameOptions(iOptionsListbox->GameOptions());
  }
  delete iOptionsListbox;
  iOptionsListbox = NULL;
  
  iAvkonAppUi->Cba()->SetCommandSetL(R_AVKON_SOFTKEYS_OPTIONS_EXIT);
  iAvkonAppUi->Cba()->DrawNow();
  iAppView->MakeVisible(ETrue);
  iAppView->DrawNow();
}
#endif

void
CMameAppUi::ScanInstalledGamesL()
{
  MameEngine().CreateZipList();

  CInstalledGameList *list =
    CInstalledGameList::NewL(MameEngine().NoOfGames());
  CleanupStack::PushL(list);
  
  CEikDialog *dialog = new (ELeave)CScanningRomDialog(MameEngine(), *list);
#ifdef __SERIES60__
  dialog->SetMopParent(this);
#endif
  TBool ok = dialog->ExecuteLD(R_DIALOG_DIPLAY_EMAME_SCANNING) == 
#ifdef __SERIES60__
                                                         EAknSoftkeyDone;
#else
                                                         EEikBidOk;
#endif
  if (ok)
  {
    MameEngine().SetInstalledGameList(*list);
    iAppView->FillInListBoxL(iGameToRun);
  }
  iAppView->DrawNow();

  CleanupStack::PopAndDestroy(list);
  MameEngine().DestroyZipList();
  
  const CInstalledGameList &engineList = MameEngine().InstalledGameList();
  if (engineList.NoOfGames() == 0)
  {
    CEikDialog *dialog = new (ELeave)
      CSimpleTextInfoDialog(R_SIMPLE_TEXT_NO_ROMS_FOUND);
    dialog->ExecuteLD(R_RICH_TEXT_DISPLAY_DIALOG);
  }
}

void CMameAppUi::DisplayAboutDialogL()
{
  CEikDialog *dialog = new (ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_ABOUT);
#if defined __SERIES60__
  dialog->SetMopParent(this);
#endif
  dialog->ExecuteLD(R_RICH_TEXT_DISPLAY_DIALOG);
}

#ifdef DUMP_GAMES_LIST_TO_FILE

void
CMameAppUi::DumpGamesListToFile()
{
  FILE *f = fopen("c:\\gameslist.txt", "w");

  for (TInt gameNo = 0; gameNo < MameEngine().NoOfGames(); gameNo++)
  {
    const struct GameDriver *game = MameEngine().GameDrivers()[gameNo];
    const struct MachineDriver *driver = game->drv;
    const struct MachineCPU *cpu = driver->cpu;
    const struct MachineSound *sound = driver->sound;

    fprintf(f, "%s\t", game->name);

    TInt i;
    float MHzSum = 0;
    for (i = 0; i < MAX_CPU; i++)
    {
      if (game->drv->cpu[i].cpu_type != 0)
      {
        TBool isAudioCpu = (cpu[i].cpu_type & CPU_AUDIO_CPU);
        
        if (strlen(MameEngine().CpuName(cpu[i].cpu_type)) == 0)
        {
          fprintf(f, "MISSING_%sCPU:%d ",
                  (isAudioCpu ? "AUDIO_" : ""),
                  cpu[i].cpu_type & ~CPU_FLAGS_MASK);
        }
        else
        {
          float freq = game->drv->cpu[i].cpu_clock;
          freq /= (1000 * 1000);
          MHzSum += freq;
          fprintf(f, "%sCPU:%s@%.2fMHz ",
                  (isAudioCpu ? "AUDIO_" : ""),
                  MameEngine().CpuName(cpu[i].cpu_type), 
                  freq);
        }
      }
    }
    fprintf(f, "MHzSum:%f ", MHzSum);

    for (i = 0; i < MAX_SOUND; i++)
    {
      if (sound[i].sound_type != 0)
      {
        if (!strcmp(sound_name(&sound[i]), "Dummy"))
        {
          fprintf(f, "MISSING_SOUND:%d ", sound[i].sound_type);
        }
        else
        {
          fprintf(f, "SOUND:%s ", sound_name(&sound[i]));
        }
      }
    }

    fprintf(f, "\t");

    fprintf(f, "%s\n", game->description);
  }

  fclose(f);
}

#endif

CMameAppView::CMameAppView(CMameEngine *aMameEngine)
  : iMameEngine(aMameEngine)
{
}

void CMameAppView::ConstructL(const TRect &aRect, TInt aGameToSelect)
{
  CreateWindowL();

  // List box setup
#if defined __SERIES60__
  iListBox = new (ELeave)CAknSingleStyleListBox;
  iListBox->SetMopParent(this);
#else
  iListBox = new (ELeave)CEikTextListBox;
#endif
// #if defined __NOKIA7710__
  iListBox->SetContainerWindowL(*this); // Should we do this for all targets?
// #endif
  iListBox->ConstructL(this);
#ifdef __SERIES60__
  iListBox->View()->SetListEmptyTextL(_L("No Mame Roms"));
#endif

  iListBox->CreateScrollBarFrameL(ETrue);
  iListBox->ScrollBarFrame()->
    SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
                            CEikScrollBarFrame::EAuto);
#if defined __UIQ__ || defined __SERIES80__
  iListBox->SetBorder(TGulBorder::EDeepSunken);
#endif
#ifndef __SERIES60__
  iListBox->SetFocus(ETrue);
#endif

#ifdef __SERIES60__
  // Find box setup
  iFindBox = CAknSearchField::NewL(*this, 
                                   CAknSearchField::EPopup,
                                   NULL,
                                   20); // Max length
  CAknFilteredTextListBoxModel *model =
    STATIC_CAST(CAknFilteredTextListBoxModel *, iListBox->Model());
  model->CreateFilterL(iListBox, iFindBox);
  iFindBox->MakeVisible( EFalse );
  iFindBox->ActivateL();
#endif

  SetRect(aRect);
#ifdef __NOKIA7710__
  SetBorder(TCknBorder::EApplicationView);
  iListBox->SetRect(Border().InnerRect(Rect()));
#endif

  FillInListBoxL(aGameToSelect);
  ActivateL();
}

CMameAppView::~CMameAppView()
{
  delete iListBox;
#ifdef __SERIES60__
  delete iFindBox;
#endif
}

TInt
CMameAppView::CountComponentControls() const
{
#ifdef __SERIES60__
  return 2; // listbox + findbox
#else
  return 1; // only listbox
#endif
}

CCoeControl *
CMameAppView::ComponentControl(TInt aIndex) const
{
  switch(aIndex)
  {
    case 0:
      return iListBox;
#ifdef __SERIES60__
    case 1:
      return iFindBox;
#endif
    default:
      return NULL;
  }
}

void 
CMameAppView::Draw(const TRect& aRect) const
{
  CWindowGc& gc = SystemGc();
  gc.SetPenStyle(CGraphicsContext::ENullPen);
  gc.SetBrushColor(KRgbWhite);
  gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
  gc.DrawRect(aRect);
}

void
CMameAppView::SizeChanged()
{
#ifdef __SERIES60__
  AknFind::HandlePopupFindSizeChanged(this, iListBox, iFindBox);
#elif defined __UIQ__ || defined __SERIES80__
  iListBox->SetRect(Rect());
#endif
}

TKeyResponse
CMameAppView::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
#ifdef __SERIES60__
  TBool needRefresh;
  if (AknFind::HandleFindOfferKeyEventL(aKeyEvent,
                                        aType,
                                        this,
                                        iListBox, 
                                        iFindBox,
                                        ETrue, // isPopup
                                        needRefresh) == EKeyWasConsumed)
  {
    if (needRefresh)
    {
      AknFind::HandlePopupFindSizeChanged(this, iListBox, iFindBox);
      DrawNow();
    }
    return EKeyWasConsumed;
  }
#endif
  return iListBox->OfferKeyEventL(aKeyEvent, aType);
}

void
CMameAppView::FillInListBoxL(TInt aGameToSelect)
{
  TBuf8<128> gameName8;
  TBuf<128> gameName;

  CDesCArray *listBoxArray=((CDesCArray*)iListBox->Model()->ItemTextArray());
  listBoxArray->Reset();
  
  TInt count = iMameEngine->InstalledGameList().NoOfGames();
  for (TInt i = 0 ;i < count; i++)
  {
    TInt gameNo = iMameEngine->InstalledGameList().GameNo(i);
    if (gameNo < iMameEngine->NoOfGames())
    {
      const struct GameDriver *game = iMameEngine->GameDrivers()[gameNo];
      FormatGamenameForListbox(gameName, gameName8, game->description);
      listBoxArray->AppendL(gameName);
    }
  }
  listBoxArray->Sort();
  iListBox->HandleItemAdditionL();

  if (aGameToSelect != -1 && aGameToSelect < iMameEngine->NoOfGames())
  {
    TInt index = 0;
    FormatGamenameForListbox(
      gameName,
      gameName8, 
      iMameEngine->GameDrivers()[aGameToSelect]->description);
    if (listBoxArray->FindIsq(gameName, index) == KErrNone)
    {
      iListBox->SetCurrentItemIndex(index);
      iListBox->View()->ScrollToMakeItemVisible(index);
      iListBox->UpdateScrollBarsL();
    }
  }
}

void
CMameAppView::FormatGamenameForListbox(TDes &name, TDes8 &name8, const char *s)
{
#ifdef __SERIES60__
  name8.Format(_L8("\t%s"), s);
#else
  name8.Format(_L8("%s"), s);
#endif
  name.Copy(name8);
}

TBool
CMameAppView::CurrentGameIndex(TInt &aIndex)
{
  TInt current = iListBox->CurrentItemIndex();
  if (current >= 0)
  {
    TPtrC ptr = iListBox->Model()->ItemText(current);
#ifdef __SERIES60__  
    aIndex = iMameEngine->GameNo(ptr.Mid(1));
#else
    aIndex = iMameEngine->GameNo(ptr);
#endif
    return ETrue;
  }
  else
  {
    return EFalse;
  }
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CMameApplication;
}

TUid
CMameApplication::AppDllUid() const
{
  return KUidEEMameApp;
}

#ifdef __SERIES60_3RD_EDITION__
TInt E32Main()
{
  return EikStart::RunApplication(NewApplication);
}
#else
GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}
#endif
