/*******************************************************************
 *
 *	File:		EMameApp.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __EMAME_APP_H
#define __EMAME_APP_H

#include "target.h"

#include <coecntrl.h>
#include <coeccntx.h>
#include <coemain.h>

#include <eikappui.h>
#include <eikapp.h>
#include <eikdoc.h>

#ifdef __SERIES60__
#include <aknapp.h>
#include <aknappui.h>
#endif
#ifdef __NOKIA7710__
#include <cknbctrl.h>
#endif

#include "eemame.hrh"

#include "EmameEngine.h"


#ifdef __SERIES60_3RD_EDITION__
const TUid KUidEEMameApp = { 0xF01FF437 };
#elif defined __SERIES60__
const TUid KUidEEMameApp = { 0x101FF433 };
#elif defined __UIQ__
const TUid KUidEEMameApp = { 0x101FF434 };
#elif defined __SERIES80__
const TUid KUidEEMameApp = { 0x101FF435 };
#elif defined __NOKIA7710__
const TUid KUidEEMameApp = { 0x101FF436 };
#else
#error No target specified
#endif


class CWsBitmap;

class CMameDocument : public CEikDocument
{
  public:
  ~CMameDocument();
  CMameDocument(CEikApplication& aApp);
  void ConstructL();
  
  CMameEngine &MameEngine()
  {
    return *iMameEngine;
  }

  protected:
    CMameEngine *iMameEngine;
  
  private:
    CEikAppUi *CreateAppUiL();
};


class CMameAppView : 
#ifdef __NOKIA7710__
                     public CCknBorderedControl
#else
                     public CCoeControl
#endif
{
  public:
    CMameAppView(CMameEngine *aMameEngine);
    ~CMameAppView();
    void ConstructL(const TRect& aRect, TInt aGameToSelect);

    TInt CountComponentControls() const;
    CCoeControl *ComponentControl(TInt aIndex) const;
    void Draw(const TRect& aRect) const;

    TBool CurrentGameIndex(TInt &aIndex);
    void FillInListBoxL(TInt aGameToSelect);

  protected:
    void SizeChanged();
    TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType);

  private:
    void FormatGamenameForListbox(TDes &name, TDes8 &name8, const char *s);

    CMameEngine *iMameEngine;
    CEikTextListBox *iListBox;
#ifdef __SERIES60__
    CAknSearchField *iFindBox;
#endif
};

class CMameAppUi : 
#ifdef __SERIES60__
                   public CAknAppUi, 
#else
                   public CEikAppUi,
#endif
		   public MGameWatcher
{
  public:
    CMameAppUi() 
      : iGameToRun(-1)
      {
      }
    
    void ConstructL();
    ~CMameAppUi();
    
    // cannot be called during construction of the document!
    static CMameAppUi *Static() 
    {
      return (CMameAppUi*) CEikonEnv::Static()->AppUi();
    }
    
    void UnblockKeys();
    
  protected:
    void HandleCommandL(TInt aCommand);
#ifdef __SERIES60__
    void GameOptionsDialogFinishedL(TInt aCommand);
#endif
    
  private:
    void LoadLastRunningGame();
    void SaveLastRunningGame();
    void RunGameL();
    void NotifyGameFinished(); 

    void ShowGameDetailsL(TInt aGame);
    void DisplayGameOptionsDialogL(const TDesC8 &aGameName);
    void ScanInstalledGamesL();
    void DisplayHelpDialogL();
    void DisplayAboutDialogL();



#ifdef DUMP_GAMES_LIST_TO_FILE
    void DumpGamesListToFile();
#endif
    CMameEngine& MameEngine()
    {
      return ((CMameDocument*) Document())->MameEngine();
    }

    TFileName iLastGameFileName;
    CGameRunner *iGameRunner;
    CEikDialog *iGameRunningDialog;
    TInt iGameToRun;
    CMameAppView* iAppView;
#ifdef __SERIES60__
    COptionsListView *iOptionsListbox;
#endif
};


class CMameApplication : 
#ifdef __SERIES60__
                         public CAknApplication
#else
                         public CEikApplication
#endif
	{
private: // from CApaApplication
	CApaDocument* CreateDocumentL();
	TUid AppDllUid() const;
	};

#endif
