/*******************************************************************
 *
 *	File:		EMameEngine.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *      Modified for Series 60 by Staffan Ulfberg (staffan@ulfberg.se)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifdef __WINS__
#include "EikEnv.h"
#endif

#include "target.h"
#include "EMameEngine.h"
#include <badesca.h>

const TInt KGameStackSize	= 0x10000;			// 64KB
const TInt KGameHeapSizeMin	= 0x10000;			// 64KB
const TInt KGameHeapSizeMax	= 0x2000000;		// 32 MB

#ifdef __SERIES60_3RD_EDITION__
_LIT(KEmameOnC, "c:\\Mame\\EEMAME");
_LIT(KOptionsFile, "c:\\Mame\\EEMAME\\options.ini");
_LIT(KGameRomsFile, "c:\\Mame\\EEMAME\\gameroms.ini");
_LIT(KDataDir, "c:\\Mame\\EEMAME\\data\\");
_LIT(KHiDir, "c:\\Mame\\EEMAME\\hi\\");
_LIT(KRomDirOnC, "C:\\Mame\\Roms\\");
_LIT(KRomDirOnD, "E:\\Mame\\Roms\\");
_LIT(KZipsOnC, "C:\\Mame\\Roms\\*.zip");
_LIT(KZipsOnD, "E:\\Mame\\Roms\\*.zip");
#elif defined __SERIES60__
_LIT(KEmameOnC, "c:\\system\\apps\\eemame\\");
_LIT(KOptionsFile, "c:\\system\\apps\\eemame\\options.ini");
_LIT(KGameRomsFile, "c:\\system\\apps\\eemame\\gameroms.ini");
_LIT(KDataDir, "C:\\system\\apps\\eemame\\data\\");
_LIT(KHiDir, "C:\\system\\apps\\eemame\\hi\\");
_LIT(KRomDirOnC, "C:\\Mame\\Roms\\");
_LIT(KRomDirOnD, "E:\\Mame\\Roms\\");
_LIT(KZipsOnC, "C:\\Mame\\Roms\\*.zip");
_LIT(KZipsOnD, "E:\\Mame\\Roms\\*.zip");
#elif defined __UIQ__
_LIT(KEmameOnC, "c:\\system\\apps\\eemame\\");
_LIT(KOptionsFile, "c:\\system\\apps\\eemame\\options.ini");
_LIT(KGameRomsFile, "c:\\system\\apps\\eemame\\gameroms.ini");
_LIT(KDataDir, "C:\\system\\apps\\eemame\\data\\");
_LIT(KHiDir, "C:\\system\\apps\\eemame\\hi\\");
_LIT(KRomDirOnC, "C:\\documents\\Media Files\\Mame\\Roms\\");
_LIT(KRomDirOnD, "D:\\Mame\\Roms\\");
_LIT(KZipsOnC, "C:\\documents\\Media Files\\Mame\\Roms\\*.zip");
_LIT(KZipsOnD, "D:\\Mame\\Roms\\*.zip");
#elif defined __SERIES80__ 
_LIT(KEmameOnC, "c:\\system\\apps\\eemame\\");
_LIT(KOptionsFile, "c:\\system\\apps\\eemame\\options.ini");
_LIT(KGameRomsFile, "c:\\system\\apps\\eemame\\gameroms.ini");
_LIT(KDataDir, "C:\\system\\apps\\eemame\\data\\");
_LIT(KHiDir, "C:\\system\\apps\\eemame\\hi\\");
_LIT(KRomDirOnC, "C:\\My files\\Mame\\Roms\\");
_LIT(KRomDirOnD, "D:\\Mame\\Roms\\");
_LIT(KZipsOnC, "C:\\My files\\Mame\\Roms\\*.zip");
_LIT(KZipsOnD, "D:\\Mame\\Roms\\*.zip");
#elif defined __NOKIA7710__
_LIT(KEmameOnC, "c:\\system\\apps\\eemame\\");
_LIT(KOptionsFile, "c:\\system\\apps\\eemame\\options.ini");
_LIT(KGameRomsFile, "c:\\system\\apps\\eemame\\gameroms.ini");
_LIT(KDataDir, "C:\\system\\apps\\eemame\\data\\");
_LIT(KHiDir, "C:\\system\\apps\\eemame\\hi\\");
_LIT(KRomDirOnC, "C:\\MyDocs\\Mame\\Roms\\");
_LIT(KRomDirOnD, "D:\\Mame\\Roms\\");
_LIT(KZipsOnC, "C:\\MyDocs\\Mame\\Roms\\*.zip");
_LIT(KZipsOnD, "D:\\Mame\\Roms\\*.zip");
#else
#error No target defined
#endif

extern "C" {
extern const struct GameDriver* drivers[];
}



CInstalledGameList* CInstalledGameList::NewL(TInt aMaxGames)
{
  CInstalledGameList *self = new(ELeave)CInstalledGameList(aMaxGames);
  CleanupStack::PushL(self);
  self->ConstuctL();
  CleanupStack::Pop(self);
  return self;
}

CInstalledGameList::CInstalledGameList(const CInstalledGameList& aOther)
{
  this->operator=(aOther);
}

CInstalledGameList& CInstalledGameList::operator=(
  const CInstalledGameList& aOther)
{
  iGames = aOther.iGames;
  Mem::Copy(this->iGameIdxs, aOther.iGameIdxs, sizeof(TInt) * iGames);
  return *this;
}

CInstalledGameList::~CInstalledGameList()
{
  delete[] iGameIdxs;
  iGameIdxs = NULL;
}

TInt
CInstalledGameList::GameNo(TInt aGameIndex) const
{
  return (aGameIndex >= 0 && aGameIndex < iGames) ?
    iGameIdxs[aGameIndex] : 0;
}

void
CInstalledGameList::AddGame(TInt aGameNo)
{
  if (iGames < iMaxGames)
  {
    iGameIdxs[iGames++] = aGameNo;
  }
}

TInt
CInstalledGameList::Externalise(RFile& aFile)
{
  TPckg<TInt> pkgVersion(KInstalledGameListFileVersion);
  TInt err = aFile.Write(pkgVersion);
  if (err != KErrNone)
  {
    return err;
  }
  TPtrC8 ptr((TUint8*)&iGames, sizeof(TInt));
  TPtrC8 ptr1((TUint8*)iGameIdxs, sizeof(TInt) * iGames);
  err = aFile.Write(ptr);
  if (err == KErrNone)
  {
    err = aFile.Write(ptr1);
  }
  return err;
}

void
CInstalledGameList::Internalise(RFile& aFile)
{
  iGames = 0;
  TInt version;
  TPckg<TInt> pkgVersion(version);
  TInt err = aFile.Read(pkgVersion);
  if (err != KErrNone || version != KInstalledGameListFileVersion)
  {
    return;
  }

  TPckg<TInt> pkgNGames(iGames);
  err = aFile.Read(pkgNGames);
  if (err != KErrNone || iGames > iMaxGames)
  {
    iGames = 0;
    return;
  }
  TPtr8 ptr1((TUint8*)iGameIdxs, sizeof(TInt) * iGames);
  if (aFile.Read(ptr1) != KErrNone)
  {
    iGames = 0;
  }
}

CInstalledGameList::CInstalledGameList(TInt aMaxGames) : iMaxGames(aMaxGames)
{
}

void CInstalledGameList::ConstuctL()
{
  iGameIdxs = new(ELeave)TInt[iMaxGames];
}



TInt CGameRunner::ThreadMameGame(TAny* aData)
{  
	CTrapCleanup* cleanup=CTrapCleanup::New();

	CGameRunner*	self = (CGameRunner*) aData;

        CActiveScheduler *scheduler = new CActiveScheduler();
        CActiveScheduler::Install(scheduler);

	TRAPD(err, self->DoRunGameL());

	delete cleanup;

	return(err);
}

CGameRunner::~CGameRunner()
{
}

CGameRunner::CGameRunner(MGameWatcher& aGameWatcher, CMameEngine& aMameEngine) : CActive(CActive::EPriorityStandard), iGameWatcher(aGameWatcher), iMameEngine(aMameEngine)
{
}

CGameRunner* CGameRunner::NewL(MGameWatcher& aGameWatcher, CMameEngine& aMameEngine)
{
	CGameRunner*	self = new(ELeave) CGameRunner(aGameWatcher, aMameEngine);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();			// self
	return self;
}

void CGameRunner::ConstructL()
{
	CActiveScheduler::Add(this);
}

void CGameRunner::RunL()
{
	iGameWatcher.NotifyGameFinished();
}

void CGameRunner::DoCancel()
{
	iThread.LogonCancel(iStatus);
}

void CGameRunner::RunGameL(TInt aGame)
{
	TBuf<32>	threadName;

		/*
		 * Hmm, there is something funny about the game thread on target. Eventhough the
		 * previous game thread has finished (we even called Terminate on it) we cannot
		 * create a new instance of it with the same name (returns with KErrAlreadyExists).
		 * Using unique names gets round that.
		 */
	iGame = aGame;

	threadName.Format(_L("Game%d"), iGameCounter++);
	User::LeaveIfError(iThread.Create(threadName, CGameRunner::ThreadMameGame,  KGameStackSize, KGameHeapSizeMin, KGameHeapSizeMax, this, EOwnerThread));

	iThread.Logon(iStatus);
	SetActive();

	iThread.Resume();
}


void CGameRunner::DoRunGameL()
{
	iMameEngine.RunGameL(iGame);
}

CMameEngine* CMameEngine::NewL(RFs& aFs)
{
	CMameEngine* self = new CMameEngine(aFs);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

CMameEngine::CMameEngine(RFs& aFs) 
  : iFs(aFs),
    iGameOptions(4),
    iGameNameKey(_FOFF(TGameOptions, iGameName), ECmpNormal8)
{
}


CMameEngine::~CMameEngine()
{
  delete iInstalledGameList;
}


void 
CMameEngine::ConstructL()
{
	iFs.MkDirAll(KEmameOnC);
	iFs.MkDirAll(KRomDirOnC);
	iFs.MkDirAll(KRomDirOnD);
        iFs.MkDirAll(KHiDir);
        iFs.MkDirAll(KDataDir);

	CalcNoOfGames();

	LoadOptionsL();

	iInstalledGameList = CInstalledGameList::NewL(iNoOfGames);
	LoadInstalledGameList();
}


void CMameEngine::KillRunningStdoutConsole()
{
#ifdef __WINS__
	TInt	id = 0;
	TBool	sentEvent = EFalse;
	RWsSession&		wsSession = CEikonEnv::Static()->WsSession();

		/*
		 * When EPOC's STDLIB shuts down it will put up a "press a key to close console".
		 * In WINS, we'll find the window group belonging to the thread STDOUT and send
		 * it a key event to have it automatically closed.
		 */
	while (id >= 0)
	{
		id = wsSession.FindWindowGroupIdentifier(id , _L("*"));
		if (id > 0)
		{
			TThreadId threadId;
			if (wsSession.GetWindowGroupClientThreadId(id , threadId) == KErrNone)
			{
				RThread	thread;
				if (thread.Open(threadId) == KErrNone)
				{
					TFullName	name;
					name = thread.Name();
					if (name.Compare(_L("STDOUT")) == 0)
					{
						TWsEvent	event;
						TKeyEvent	key;
						
						key.iCode=EKeyEscape;
						key.iScanCode=0;
						key.iModifiers=0;

						event.SetType(EEventKey);
						*event.Key()=key;
						event.SetTimeNow();
						wsSession.SendEventToWindowGroup(id, event);

						sentEvent = ETrue;
					}
					thread.Close();
				}
			}
		}
	}
	if (!sentEvent)
		User::InfoPrint(_L("Please close the task named STDOUT"));
#endif
}



void CMameEngine::RunGameL(TInt aGame)
{
  const struct GameDriver *game = drivers[aGame];
  TBuf8<8> text8;
  text8.Copy((const TUint8*)game->name);
  ::RunGame(aGame, GameOptions(text8), iAppWindowGroupId);
}



void CMameEngine::SetWindowGroupId(TInt aWindowGroupId)
{
	iAppWindowGroupId = aWindowGroupId;
}


TInt CMameEngine::GameNo(const TDesC& aGameName)
{
	TInt	gameNo = 0;
	const struct GameDriver**	game = drivers;
	while(*game)
	{
		TPtrC8	gameName8((TUint8*) (*game)->description);
		TBuf<128>	gameName;
		gameName.Copy(gameName8);
		if (aGameName.Compare(gameName) == 0)
			break;
		game++;
		gameNo++;
	}
	return gameNo;
}

TBool CMameEngine::IsGameCloneOf(const struct GameDriver* aGame)
{
	return (aGame->clone_of && !(aGame->clone_of->flags & NOT_A_DRIVER));
}

const char* CMameEngine::GameName(TInt aGameNo)
{
	const struct GameDriver**	game = drivers;
	return (*(game+aGameNo))->description;
}

const char* CMameEngine::GameShortName(TInt aGameNo)
{
	const struct GameDriver**	game = drivers;
	return (*(game+aGameNo))->name;
}


void CMameEngine::ForceGameExit()
{
	gForcedExitFromFrontEnd = 1;
}


void CMameEngine::ResumeGame()
{
	SetResumeFromFrontEnd(ETrue);
}

TGameOptions 
CMameEngine::DefaultOptions()
{
  TGameOptions gameOptions;
#if defined __UIQ__ || defined __NOKIA7710__
  gameOptions.iVirtualKeyboardEnabled = ETrue;
#endif
  gameOptions.iSoundEnabled = ETrue;
  gameOptions.iGameSize = TGameOptions::EGameSizeFitScreen;
  gameOptions.iKeepAspectRatio = ETrue;
  gameOptions.iScreenOrientation = TGameOptions::EScreenOrientationNormal;
  gameOptions.iSoundVolume = 3; // 1 -- 10
  gameOptions.iFrameSkip = 6;
  gameOptions.iAutoFrameSkipEnabled = ETrue;
  gameOptions.iMaxAutoFrameSkip = 9;
  gameOptions.iThrottleSpeedEnabled = ETrue;

  return gameOptions;
}

void 
CMameEngine::LoadOptionsL()
{
  RFile	file;
  TInt err;

  iGameOptions.Reset();
  err = file.Open(iFs, KOptionsFile, 0);
  if (err == KErrNone)
  {
    TInt version;
    TPckg<TInt> pkgVersion(version);
    err = file.Read(pkgVersion);

    if (err == KErrNone && version != KOptionsFileVersion)
    {
      err = -1;
    }
    if (err == KErrNone)
    {
      TGameOptions gameOptions;
      TPckg<TGameOptions> pkg(gameOptions);
      do
      {
        err = file.Read(pkg);
        if (err == KErrNone && pkg.Length() != 0)
        {
          TRAP(err, iGameOptions.AppendL(gameOptions));
        }
      }
      while (err == KErrNone && pkg.Length() != 0);
    }
    file.Close();
  }

  TGameOptions defaultOptions(DefaultOptions());
  defaultOptions.iGameName = KDefaultOptionsGameName;
  TInt index;
  if (iGameOptions.Find(defaultOptions, iGameNameKey, index))
  {
    iGameOptions.AppendL(defaultOptions);
  }
}

TInt 
CMameEngine::SaveOptions()
{
  RFile	file;
  TInt err;
  err = file.Replace(iFs, KOptionsFile, EFileWrite);
  if (err == KErrNone)
  {
    TPckgC<TInt> pkgVersion(KOptionsFileVersion);
    err = file.Write(pkgVersion);

    TInt count = iGameOptions.Count();
    TInt i = 0;
    TGameOptions gameOptions;
    TPckgC<TGameOptions> pkg(gameOptions);
    while (err == KErrNone && i < count)
    {
      gameOptions = iGameOptions[i];
      err = file.Write(pkg);
      i++;
    }
    file.Close();
  }
  return err;
}

void
CMameEngine::SetGameOptions(const TGameOptions &aGameOptions)
{ 
  TInt index;
  if (iGameOptions.Find(aGameOptions, iGameNameKey, index))
  {
    iGameOptions.AppendL(aGameOptions); // Not found -- add
  }
  else
  {
    iGameOptions[index] = aGameOptions; // Found -- update
  }
  SaveOptions();
}

void
CMameEngine::ClearGameOptions(const TDesC8 &aGameName)
{
  if (aGameName == KDefaultOptionsGameName)
  {
    return; // Not permitted
  }
  TGameOptions gameOptions = DefaultOptions();
  gameOptions.iGameName = aGameName;
  TInt index;
  if (iGameOptions.Find(gameOptions, iGameNameKey, index))
  {
    return; // Not found
  }
  iGameOptions.Delete(index);
  SaveOptions();
}

TBool
CMameEngine::GameHasSpecificOptions(const TDesC8 &aGameName)
{
  TGameOptions gameOptions = DefaultOptions();
  gameOptions.iGameName = aGameName;
  TInt index;
  return iGameOptions.Find(gameOptions, iGameNameKey, index) == 0;
}

TGameOptions
CMameEngine::GameOptions(const TDesC8 &aGameName)
{
  TGameOptions gameOptions = DefaultOptions();
  gameOptions.iGameName = aGameName;
  TInt index;
  if (iGameOptions.Find(gameOptions, iGameNameKey, index))
  {
    gameOptions.iGameName = KDefaultOptionsGameName;
    if (iGameOptions.Find(gameOptions, iGameNameKey, index))
    {
      ASSERT(EFalse);
    }
  }
  return iGameOptions[index];
}

const struct GameDriver** CMameEngine::GameDrivers()
{
	return drivers;
}

const char* CMameEngine::CpuName(TInt aCpuType)
{
  return cputype_name(aCpuType);
}

const CInstalledGameList& CMameEngine::InstalledGameList()
{
	return *iInstalledGameList;
}

TInt CMameEngine::SaveInstalledGameList()
{
	RFile	file;
	TInt err = file.Open(iFs, KGameRomsFile, EFileWrite);
	if (err)
		err = file.Create(iFs, KGameRomsFile, EFileWrite);
	if (!err)
	{
		err = iInstalledGameList->Externalise(file);
		file.Close();
	}
	return err;
}

void CMameEngine::LoadInstalledGameList()
{
	RFile	file;
	TInt err = file.Open(iFs, KGameRomsFile, 0);
	if (!err)
	{
          iInstalledGameList->Internalise(file);
          file.Close();
	}
}

void CMameEngine::SetInstalledGameList(const CInstalledGameList& aInstalledGameList)
{
	*iInstalledGameList = aInstalledGameList;
	if (SaveInstalledGameList() == KErrNone)
	{
		if (iInstalledGameList->NoOfGames() > 0)
			User::InfoPrint(_L("Game list saved"));
	}
}


void CMameEngine::CalcNoOfGames()
{
	iNoOfGames = 0;
	const struct GameDriver**	game = GameDrivers();
	while(*game++)
		iNoOfGames++;
}

void DestroyCachedZipList();
void SetCachedZipList(CDesCArrayFlat* aCachedZipList);

void CMameEngine::DestroyZipList()
{
	DestroyCachedZipList();
}

void CMameEngine::CreateZipList()
{
	CDesCArrayFlat* desArray = new(ELeave)CDesCArrayFlat(10);
	CleanupStack::PushL(desArray);
	CDir*	entryList = NULL;
	if (iFs.GetDir(KZipsOnC, 0, 0, entryList) == KErrNone)
	{
		CleanupStack::PushL(entryList);
		for (TInt i=0 ; i<entryList->Count() ; i++)
		{
			const TEntry& entry = (*entryList)[i];
			TParsePtrC	parse(entry.iName);
			TPtrC	name(parse.Name());
			desArray->AppendL(name);
		}
		CleanupStack::PopAndDestroy(entryList);
	}
	entryList = NULL;
	if (iFs.GetDir(KZipsOnD, 0, 0, entryList) == KErrNone)
	{
		CleanupStack::PushL(entryList);
		for (TInt i=0 ; i<entryList->Count() ; i++)
		{
			const TEntry& entry = (*entryList)[i];
			TParsePtrC	parse(entry.iName);
			TPtrC	name(parse.Name());
			desArray->AppendL(name);
		}
		CleanupStack::PopAndDestroy(entryList);
	}
	CleanupStack::Pop();

	SetCachedZipList(desArray);
}

extern "C" {
#include "audit.h"
}
/*
 * Code stolen from audit.c.  This is essentially VerifyRomSet()
 * except that there's no printing here.
 */
TBool
CMameEngine::CheckIfGameRomsComplete(int game)
{
  tAuditRecord *aud;
  int count;
  int archive_status = 0;
  const struct GameDriver *gamedrv = drivers[game];

  if ((count = AuditRomSet (game, &aud)) == 0)
    return EFalse;
  
  if (count == -1) return ETrue;
  
  if (gamedrv->clone_of)
  {
    int i;
    int cloneRomsFound = 0;
    
    /* count number of roms found that are unique to clone */
    for (i = 0; i < count; i++)
      if (aud[i].status != AUD_ROM_NOT_FOUND)
        if (!RomInSet (gamedrv->clone_of, aud[i].expchecksum))
          cloneRomsFound++;
    
#ifndef MESS
    /* Different MESS systems can use the same ROMs */
    if (cloneRomsFound == 0)
      return EFalse;
#endif
  }
  
  while (count--)
  {
    archive_status |= aud->status;
    aud++;
  }
  
  if (archive_status &
      (AUD_ROM_NOT_FOUND|AUD_BAD_CHECKSUM|AUD_MEM_ERROR|AUD_LENGTH_MISMATCH))
		return EFalse;
  if (archive_status & 
      (AUD_ROM_NEED_DUMP|AUD_ROM_NEED_REDUMP|AUD_NOT_AVAILABLE))
    return ETrue;

  return ETrue;
}

