/*******************************************************************
 *
 *	File:		EMameEngine.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __EMAME_ENGINE_H
#define __EMAME_ENGINE_H

#include <e32std.h>
#include <e32base.h>

#include "EMame.h"
#include "EMameGlobal.h"

#if defined FULL_EEMAME
const TInt KInstalledGameListFileVersion = 2001;
#elif defined MINI_EEMAME
const TInt KInstalledGameListFileVersion = 2002;
#elif defined MICRO_EEMAME
const TInt KInstalledGameListFileVersion = 2003;
#else
#error "FULL_EEMAME, MINI_EEMAME, or MICRO_EEMAME must be defined"
#endif

class CInstalledGameList : public CBase
{
  public:
    static CInstalledGameList* NewL(TInt aMaxGames);
    CInstalledGameList(const CInstalledGameList& aOther);
    CInstalledGameList& operator=(const CInstalledGameList& aOther);
    ~CInstalledGameList();
    
    TInt NoOfGames() const {return iGames;}
    TInt GameNo(TInt aGameIndex) const;
    void AddGame(TInt aGameNo);
    TInt Externalise(RFile& aFile);
    void Internalise(RFile& aFile);
    
  protected:
    CInstalledGameList(TInt aMaxGames);
    void ConstuctL();
    TInt	iGames;
    TInt	iMaxGames;
    TInt*	iGameIdxs;
};

class CMameEngine;

class MGameWatcher
{
public:
	virtual void NotifyGameFinished() = 0;
};


class CGameRunner : public CActive
{
public:
	static CGameRunner* NewL(MGameWatcher& aGameWatcher, CMameEngine& aMameEngine);
	~CGameRunner();

	void RunGameL(TInt aGame);

protected:
	CGameRunner(MGameWatcher& aGameWatcher, CMameEngine& aMameEngine);
	void ConstructL();

	virtual void RunL();
	virtual void DoCancel();

	void DoRunGameL();
	static TInt ThreadMameGame(TAny* aData);
	
	MGameWatcher&		iGameWatcher;
	RThread				iThread;
	TInt				iGame;
	TInt				iGameCounter;
	CMameEngine&		iMameEngine;
};




class CMameEngine : public CBase
{
  public:
    static CMameEngine* NewL(RFs& aFs);
    ~CMameEngine();
    
    TGameOptions GameOptions(const TDesC8 &aGameName);
    void SetGameOptions(const TGameOptions& aGameOptions);
    void ClearGameOptions(const TDesC8 &aGameName);
    TBool GameHasSpecificOptions(const TDesC8 &aGameName);

	void						SetWindowGroupId(TInt aWindowGroupId);
	void						ShutdownGame();
	const struct GameDriver**	GameDrivers();
	const char*					CpuName(TInt aCpuType);
	TInt						GameNo(const TDesC& aGameName);
	const char*					GameName(TInt aGameNo);
	const char*					GameShortName(TInt aGameNo);
	TBool						IsGameCloneOf(const struct GameDriver* aGame);
	void						ForceGameExit();
	void						ResumeGame();
	const CInstalledGameList&	InstalledGameList();
	void						SetInstalledGameList(const CInstalledGameList& aInstalledGameList);
	TInt						NoOfGames(){return iNoOfGames;}

	void CreateZipList();
	void DestroyZipList();

        static TBool CheckIfGameRomsComplete(int game);

protected:
	CMameEngine(RFs& aFs);
	void ConstructL();

	void LoadOptionsL();
	TInt SaveOptions();
	TGameOptions DefaultOptions();

	void LoadInstalledGameList();
	TInt SaveInstalledGameList();

	void CalcNoOfGames();

	friend class CGameRunner;
	void RunGameL(TInt aGame);
	void KillRunningStdoutConsole();

	RFs &iFs;

	CArrayFixFlat<TGameOptions> iGameOptions;
	TKeyArrayFix iGameNameKey;

	TInt				iAppWindowGroupId;
	CInstalledGameList*	iInstalledGameList;
	TInt				iNoOfGames;

};


#endif			/* __EMAME_ENGINE_H */
