/*
 * From xmame-0.37: sc/unix/frameskip_drivers/dos.c
 *
 * Slightly modified for EMame for Series 60.
 */

#include <e32std.h>
extern "C" {
#include "osdepend.h"
#include "driver.h"
#include "profiler.h"
}

int frameskip = 6;
int throttle = 1;
int autoframeskip = 0;
int video_fps;
int max_autoframeskip = 11;
static int showfps = 0;
static int showfpstemp = 0;

#define FRAMESKIP_LEVELS 12
typedef int uclock_t;
#define uclock() (User::TickCount() * 10000)

#if defined __WINS__ && !defined __SERIES60_3RD_EDITION__
#define UCLOCKS_PER_SEC (10 * 10000)
#else
#define UCLOCKS_PER_SEC (64 * 10000)
#endif

static int frameskip_counter = 0;
static int speed=100;

int dos_skip_next_frame(struct osd_bitmap *bitmap)
{
	static const int skiptable[FRAMESKIP_LEVELS][FRAMESKIP_LEVELS] =
	{
		{ 0,0,0,0,0,0,0,0,0,0,0,0 },
		{ 0,0,0,0,0,0,0,0,0,0,0,1 },
		{ 0,0,0,0,0,1,0,0,0,0,0,1 },
		{ 0,0,0,1,0,0,0,1,0,0,0,1 },
		{ 0,0,1,0,0,1,0,0,1,0,0,1 },
		{ 0,1,0,0,1,0,1,0,0,1,0,1 },
		{ 0,1,0,1,0,1,0,1,0,1,0,1 },
		{ 0,1,0,1,1,0,1,0,1,1,0,1 },
		{ 0,1,1,0,1,1,0,1,1,0,1,1 },
		{ 0,1,1,1,0,1,1,1,0,1,1,1 },
		{ 0,1,1,1,1,1,0,1,1,1,1,1 },
		{ 0,1,1,1,1,1,1,1,1,1,1,1 }
	};
	static const int waittable[FRAMESKIP_LEVELS][FRAMESKIP_LEVELS] =
	{
		{ 1,1,1,1,1,1,1,1,1,1,1,1 },
		{ 2,1,1,1,1,1,1,1,1,1,1,0 },
		{ 2,1,1,1,1,0,2,1,1,1,1,0 },
		{ 2,1,1,0,2,1,1,0,2,1,1,0 },
		{ 2,1,0,2,1,0,2,1,0,2,1,0 },
		{ 2,0,2,1,0,2,0,2,1,0,2,0 },
		{ 2,0,2,0,2,0,2,0,2,0,2,0 },
		{ 2,0,2,0,0,3,0,2,0,0,3,0 },
		{ 3,0,0,3,0,0,3,0,0,3,0,0 },
		{ 4,0,0,0,4,0,0,0,4,0,0,0 },
		{ 6,0,0,0,0,0,6,0,0,0,0,0 },
		{12,0,0,0,0,0,0,0,0,0,0,0 }
	};
	int i;
	uclock_t curr;
	static uclock_t prev_frames[FRAMESKIP_LEVELS]={0,0,0,0,0,0,0,0,0,0,0,0};
	static uclock_t prev=0;

	/* now wait until it's time to update the screen */
	if (skiptable[frameskip][frameskip_counter] == 0)
	{
        	if (throttle)
        	{
        		uclock_t target,target2;
        		profiler_mark(PROFILER_IDLE);

        		/* wait until enough time has passed since last frame... */
        		target = prev +
        			waittable[frameskip][frameskip_counter] * UCLOCKS_PER_SEC/video_fps;

        		/* ... OR since FRAMESKIP_LEVELS frames ago. This way, if a frame takes */
        		/* longer than the allotted time, we can compensate in the following frames. */
        		target2 = prev_frames[frameskip_counter] +
        			FRAMESKIP_LEVELS * UCLOCKS_PER_SEC/video_fps;

        		if (target - target2 > 0) target = target2;
        		
        		curr = uclock();
        		
        		/* If we need to sleep more then half a second,
        		   we've somehow got totally out of sync. So
        		   if this happens we reset all counters */
        		if ((target - curr) > (UCLOCKS_PER_SEC / 2))
        		   for (i=0; i < FRAMESKIP_LEVELS; i++)
        		      prev_frames[i] = curr;
        		else
        		   while ((curr - target) < 0)
        		   {
        		      curr = uclock();
        		   }

        		profiler_mark(PROFILER_END);
        	}
        	else curr = uclock();

        	if (frameskip_counter == 0 && (curr - prev_frames[frameskip_counter]))
        	{
        		int divdr;

        		divdr = video_fps * (curr - prev_frames[frameskip_counter]) / (100 * FRAMESKIP_LEVELS);
        		speed = (UCLOCKS_PER_SEC + divdr/2) / divdr;
        	}

        	prev = curr;
        	for (i = 0;i < waittable[frameskip][frameskip_counter];i++)
        		prev_frames[(frameskip_counter + FRAMESKIP_LEVELS - i) % FRAMESKIP_LEVELS] = curr;

        	if (showfps || showfpstemp)
        	{
        		int fps;
        		char buf[30];
        		int divdr;

        		divdr = 100 * FRAMESKIP_LEVELS;
        		fps = (video_fps * (FRAMESKIP_LEVELS - frameskip) * speed + (divdr / 2)) / divdr;
        		sprintf(buf,"%s%s%2d %3d%%(%2d/%d fps)", throttle?"T ":"",
			(throttle && autoframeskip)?"autofskp":"fskp",
			frameskip,speed,fps,(int)video_fps);
        		ui_text(bitmap, buf, Machine->uiwidth-strlen(buf)*Machine->uifontwidth, 0);
        	}
        	
		if (throttle && autoframeskip && frameskip_counter == 0)
		{
			static int frameskipadjust;

			if (speed >= 97)
			{
				frameskipadjust++;
				if (frameskipadjust >= 3)
				{
					frameskipadjust = 0;
					if (frameskip > 0) frameskip--;
				}
			}
			else
			{
				if (speed < 80)
					frameskipadjust -= (90 - speed) / 5;
				else
				{
					/* don't push frameskip too far if we are close to 100% speed */
					if (frameskip < 8)
						frameskipadjust--;
				}

				while (frameskipadjust <= -2)
				{
					frameskipadjust += 2;
					if (frameskip < max_autoframeskip) frameskip++;
				}
			}
		}
	}
	
	frameskip_counter = (frameskip_counter + 1) % FRAMESKIP_LEVELS;
	
	return skiptable[frameskip][frameskip_counter];
}

void
dos_change_framskip_from_keys(struct osd_bitmap *bitmap)
{
  if (input_ui_pressed(IPT_UI_FRAMESKIP_INC))
  {
    if (autoframeskip)
    {
      autoframeskip = 0;
      frameskip = 0;
    }
    else
    {
      if (frameskip == FRAMESKIP_LEVELS-1)
      {
        frameskip = 0;
        autoframeskip = 1;
      }
      else
        frameskip++;
    }
    
    if (showfps == 0)
      showfpstemp = 2*video_fps*(12-frameskip)/12;
  }
  
  if (input_ui_pressed(IPT_UI_FRAMESKIP_DEC))
  {
    if (autoframeskip)
    {
      autoframeskip = 0;
      frameskip = FRAMESKIP_LEVELS-1;
    }
    else
    {
      if (frameskip == 0)
        autoframeskip = 1;
      else
        frameskip--;
    }
    
    if (showfps == 0)
      showfpstemp = 2*video_fps*(12-frameskip)/12;
  }
  
  if (input_ui_pressed(IPT_UI_THROTTLE))
  {
    throttle ^= 1;
    if (showfps == 0)
      showfpstemp = 2*video_fps*(12-frameskip)/12;
  }

  int need_to_clear_bitmap = 0;

  if (input_ui_pressed(IPT_UI_SHOW_FPS))
  {
    if (showfpstemp)
    {
      showfpstemp = 0;
      need_to_clear_bitmap = 1;
    }
    else
    {
      showfps ^= 1;
      if (showfps == 0)
      {
        need_to_clear_bitmap = 1;
      }
    }
  }

  if (showfpstemp)
  {
    showfpstemp--;
    if (showfpstemp == 0) need_to_clear_bitmap = 1;
  }

  if (need_to_clear_bitmap) 
  {
    //schedule_full_refresh();
    osd_clearbitmap(bitmap);
  }
}
