/*******************************************************************
 *
 *	File:		EMame.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

extern "C" {
#include "osdepend.h"
#include "mame.h"
#include "driver.h"
}

#undef NULL

#include <stdio.h>
#include <unistd.h>
#include <e32std.h>
#include <f32file.h>
#include <e32svr.h>
#include <estlib.h>
#include <eikdll.h>
#include <w32std.h>

#include "cpuintrf.h"

#include "target.h"
#include "emame.h"
#include "audio.h"		// for sample rate
#include "EMameGlobal.h"

static TGameOptions		sGameOptions;

const TGameOptions& GameOptions()
{
	return sGameOptions;
}

static void
SetDefaultGamesOptions(TSize aScreenSize)
{
    options.mame_debug = 0;
    options.record = 0;
    options.playback = 0;
    options.cheat = 0;
    options.gui_host = 1;
    
    options.samplerate = 8000;
    options.use_samples = 1;
    options.use_emulated_ym3812 = 0;
    
    options.color_depth = 8;
    options.vector_width = aScreenSize.iWidth;
    options.vector_height = aScreenSize.iHeight;
    options.debug_width = aScreenSize.iWidth;
    options.debug_height = aScreenSize.iHeight;
    
    options.norotate = 0;
    options.ror = 0;
    options.rol = 0;
    options.flipx = 0;
    options.flipy = 0;
    
    options.translucency = 0;
    options.antialias = 0;
    options.use_artwork = 0;
    
    options.beam = 0x00010000;
    options.flicker = 0;
}

void RunGameL(TInt aGame)
{
	TInt err = run_game(aGame); // This can leave!
	
	if (err)
	{
			/*
			 * "Normal" game error usually because of roms not being present
			 */
		printf("Press any key to continue\n");
		getchar();
	}
}


void
RunGame(TInt aGame, const TGameOptions& aGameOptions, TInt aAppWindowGroupId)
{
#ifdef __WINS__X
  int argc=0;
  char** argv=0;
  char** envp=0;
  
  __crt0(argc,argv,envp);
#endif
  
  gForcedExitFromFrontEnd = 0;
  SetResumeFromFrontEnd(EFalse);
  
#if defined __SERIES60__
  TSize screenSize(176, 208);
#elif defined __UIQ__
  TSize screenSize(208, 320);
#elif defined __SERIES80__
  TSize screenSize(640, 200);
#elif defined __NOKIA7710__
  TSize screenSize(640, 320);
#endif
  
  SetDefaultGamesOptions(screenSize);
  sGameOptions = aGameOptions;
  switch (sGameOptions.iScreenOrientation)
  {
    case TGameOptions::EScreenOrientationRotateRight:
      options.rol = 1;
      break;
    case TGameOptions::EScreenOrientationUpsideDown:
      options.flipx = 1;
      options.flipy = 1;
      break;
    case TGameOptions::EScreenOrientationRotateLeft:
      options.ror = 1;
      break;
  }
  
  if (!sGameOptions.iSoundEnabled)
  {
    options.samplerate = 0;
  }
  
  TRAPD(err, RunGameL(aGame));
  if (err)
  {
    Oops();
  }
  
#ifdef __WINS__X
  fclose(stdin);
  fclose(stdout);
  fclose(stderr);
#endif
}
